from plone.testing import zca
from plone.testing import Layer
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import applyProfile
from plone.app.testing import setRoles, TEST_USER_ID, TEST_USER_NAME, login
from plone.testing import z2
from zope.configuration import xmlconfig


class LatexZCMLLayer(Layer):
    """A layer which only sets up the zcml, but does not start a zope
    instance.
    """

    defaultBases = (zca.ZCML_DIRECTIVES, )

    def testSetUp(self):
        import ftw.book.tests
        self['configurationContext'] = zca.stackConfigurationContext(
            self.get('configurationContext'))

        xmlconfig.file('latex.zcml', ftw.book.tests,
                       context=self['configurationContext'])

    def testTearDown(self):
        del self['configurationContext']


LATEX_ZCML_LAYER = LatexZCMLLayer()


class FtwBookLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE, )

    def setUpZope(self, app, configurationContext):
        # Load ZCML
        import ftw.book
        import ftw.book.portlets
        import simplelayout.base
        import simplelayout.types.common

        xmlconfig.file('configure.zcml', ftw.book,
                       context=configurationContext)
        xmlconfig.file('configure.zcml', ftw.book.portlets,
                       context=configurationContext)

        xmlconfig.file('configure.zcml', simplelayout.base,
                       context=configurationContext)
        xmlconfig.file('configure.zcml', simplelayout.types.common,
                       context=configurationContext)

        # installProduct() is *only* necessary for packages outside
        # the Products.* namespace which are also declared as Zope 2
        # products, using <five:registerPackage /> in ZCML.
        z2.installProduct(app, 'ftw.book')
        z2.installProduct(app, 'simplelayout.base')
        z2.installProduct(app, 'simplelayout.types.common')

    def setUpPloneSite(self, portal):
        # Install into Plone site using portal_setup
        applyProfile(portal, 'ftw.book:default')
        applyProfile(portal, 'simplelayout.base:default')
        applyProfile(portal, 'simplelayout.types.common:default')

        setRoles(portal, TEST_USER_ID, ['Manager'])
        login(portal, TEST_USER_NAME)


FTW_BOOK_FIXTURE = FtwBookLayer()
FTW_BOOK_INTEGRATION_TESTING = IntegrationTesting(
    bases=(FTW_BOOK_FIXTURE, ), name="FtwBook:Integration")



from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import registerType
from ftw.book.config import PROJECTNAME
from ftw.book.interfaces import IChapter
from simplelayout.base.interfaces import ISimpleLayoutBlock
from simplelayout.base.interfaces import ISimpleLayoutCapable
from simplelayout.types.common.content.page import Page
from zope.interface import implements


class Chapter(Page):
    implements(IChapter, ISimpleLayoutCapable, ISimpleLayoutBlock)
    security = ClassSecurityInfo()


registerType(Chapter, PROJECTNAME)



from ftw.book.helpers import BookHelper
from simplelayout.types.common.browser.views import BlockView


class BookBlockView(BlockView):

    def get_dynamic_title(self):
        return BookHelper()(self.context)


class BookChapterView(BlockView):

    def get_dynamic_title(self):
        return BookHelper()(self.context, linked=True)



from Acquisition import aq_inner, aq_parent
from Products.ATContentTypes.lib.imagetransform import ATCTImageTransform
from ftw.book.interfaces import IBook
from ftw.pdfgenerator.html2latex.utils import generate_manual_caption
from plone.app.layout.navigation.interfaces import INavigationRoot


HEADING_COMMANDS = [
    'chapter',
    'section',
    'subsection',
    'subsubsection',
    'paragraph',
    'subparagraph',
    ]


IMAGE_LAYOUTS = {
    'no-image': None,

    'small': {
        'width': '0.25',
        'align': 'l'},

    'middle': {
        'width': '0.5',
        'align': 'l'},

    'full': {
        'width': '',
        'align': ''},

    'middle-right': {
        'width': '0.5',
        'align': 'r'},

    'small-right': {
        'width': '0.25',
        'align': 'r'},
    }


def get_latex_heading(context, layout, toc=True):
    title = layout.get_converter().convert(context.pretty_title_or_id())

    # level: depth of rendering
    level = -1
    max_level = len(HEADING_COMMANDS) - 1

    obj = context
    while obj and not IBook.providedBy(obj) and level != max_level:
        obj = aq_parent(aq_inner(obj))
        level += 1

        if INavigationRoot.providedBy(obj):
            # cancel, use section
            level = 1
            break

    command = HEADING_COMMANDS[level]

    # generate latex
    tocmark = ''
    if not toc:
        tocmark = '*'

    latex = '\%s%s{%s}\n' % (
        command,
        tocmark,
        title)

    return latex


def get_raw_image_data(image):
    # XXX use scaling?
    transformer = ATCTImageTransform()
    img = transformer.getImageAsFile(img=image)

    if img is not None:
        return img.read()

    elif isinstance(image.data, str):
        return image.data

    else:
        return image.data.read()


class ImageLaTeXGenerator(object):
    """Generates LaTeX code for including images. Optimized for simplelayout
    image layoutes.
    """

    def __init__(self, context, layout):
        """Arguments:
        context -- object where the image is stored on (portal_type Image
        for instance).
        layout -- ILaTeXLayout object.
        """
        self.layout = layout
        self.context = context

    def __call__(self, image, image_layout, floatable=False, caption=False):
        """Arguments:
        image -- The image object, containing the image data.
        image_layout -- Simplelayout image style (e.g. "small" or
        "middle-right").
        floatable -- If there is normal text right after the image, it is
        possible to float it with ``floatable=True``.
        caption -- The caption of the image.
        """
        if image_layout == 'no-image':
            return ''

        width_ratio = self._get_width_ration_by_image_layout(image_layout)
        align = self._get_alignment_by_image_layout(image_layout)
        return self.render(image, width_ratio, align, floatable=floatable,
                           caption=caption)

    def render(self, image, width_ratio, alignment, floatable=False,
               caption=False):
        """Arguments:
        image -- The image object, containing the image data.
        width_ratio -- The LaTeX style ratio (compared with \textwidth) of
        the width of the image (e.g. '0.5').
        alignment -- Alignment ('l', 'c' or 'r').
        floatable -- If there is normal text right after the image, it is
        possible to float it with ``floatable=True``.
        caption -- The caption of the image.
        """

        width = r'%s\textwidth' % width_ratio
        latex = self._generate_includegraphics_latex(image, width)
        latex = self._extend_latex_with_caption(latex, caption, floatable)

        if floatable:
            latex = self._extend_latex_with_floating(latex, alignment, width)
        else:
            latex = self._extend_latex_with_alignment(latex, alignment)

        return latex

    def _get_width_ration_by_image_layout(self, image_layout):
        if image_layout in IMAGE_LAYOUTS:
            return IMAGE_LAYOUTS[image_layout]['width']
        else:
            return ''

    def _get_alignment_by_image_layout(self, image_layout):
        if image_layout in IMAGE_LAYOUTS:
            return IMAGE_LAYOUTS[image_layout]['align']
        else:
            return 'l'

    def _get_image_filename(self):
        return '%s_image' % self.context.UID()

    def _generate_includegraphics_latex(self, image, width):
        name = self._get_image_filename()

        self.layout.use_package('graphicx')
        self.layout.get_builder().add_file(
            '%s.jpg' % name, get_raw_image_data(image))

        return r'\includegraphics[width=%s]{%s}' % (width, name)

    def _extend_latex_with_caption(self, latex, caption, floatable):
        if not caption:
            return latex

        caption = self.layout.get_converter().convert(caption)
        if floatable:
            return '%s\n\\caption{%s}' % (latex, caption)

        else:
            return '%s\n%s' % (
                latex,
                generate_manual_caption(caption, 'figure'))

    def _extend_latex_with_alignment(self, latex, alignment):
        if alignment == 'c':
            return '\n'.join((
                    r'\begin{center}',
                    latex,
                    r'\end{center}'))

        elif alignment == 'r':
            return '\n'.join((
                    r'\begin{flushright}',
                    latex,
                    r'\end{flushright}'))

        else:
            return latex

    def _extend_latex_with_floating(self, latex, alignment, width):
        self.layout.use_package('wrapfig')
        return '\n'.join((
                r'\begin{wrapfigure}{%s}{%s}' % (alignment, width),
                latex,
                r'\end{wrapfigure}'))



from ftw.book.interfaces import ILaTeXCodeInjectionEnabled
from ftw.book.interfaces import IWithinBookLayer
from ftw.book.testing import LATEX_ZCML_LAYER
from ftw.pdfgenerator.layout.baselayout import BaseLayout
from plone.mocktestcase import MockTestCase
from zope.interface import directlyProvides


class TestInjectionAwareConvertObject(MockTestCase):

    layer = LATEX_ZCML_LAYER

    def setUp(self):
        super(TestInjectionAwareConvertObject, self).setUp()

        context = self.create_dummy()
        request = self.create_dummy()
        directlyProvides(request, IWithinBookLayer)
        builder = self.create_dummy()

        self.layout = BaseLayout(context, request, builder)

    def test_not_injected_without_interface(self):
        obj = self.mocker.mock()
        self.expect(obj.Schema()).count(0)

        self.replay()

        self.assertEqual(self.layout.render_latex_for(obj), '')

    def test_injected_with_interface(self):
        latex_pre_code = 'INJECTED PRE LATEX CODE'
        latex_post_code = 'INJECTED POST LATEX CODE'

        obj_dummy = self.create_dummy()
        directlyProvides(obj_dummy, ILaTeXCodeInjectionEnabled)
        obj = self.mocker.proxy(obj_dummy, spec=None)
        schema = self.mocker.mock()
        self.expect(obj.Schema()).result(schema).count(2)

        self.expect(schema.getField('preLatexCode').get(obj)).result(
            latex_pre_code)
        self.expect(schema.getField('postLatexCode').get(obj)).result(
            latex_post_code)

        self.expect(obj.getPhysicalPath()).result(
            ['', 'myobj']).count(3)  # 3 = pre + post + assertion below

        self.replay()
        latex = self.layout.render_latex_for(obj)

        self.assertIn(latex_pre_code, latex)
        self.assertIn(latex_post_code, latex)
        self.assertIn('/'.join(obj.getPhysicalPath()), latex)

    def test_bad_schemaextender_state(self):
        # sometimes the field can not be retrieved. We do nothing and we
        # don't fail in this case.
        obj_dummy = self.create_dummy()
        directlyProvides(obj_dummy, ILaTeXCodeInjectionEnabled)
        obj = self.mocker.proxy(obj_dummy, spec=None)
        schema = self.mocker.mock()
        self.expect(obj.Schema()).result(schema).count(2)

        # pre latex: field not found
        self.expect(schema.getField('preLatexCode')).result(None)

        # post latex: field value is empty
        self.expect(schema.getField('postLatexCode').get(obj)).result('')

        self.replay()
        latex = self.layout.render_latex_for(obj)

        self.assertEqual(latex.strip(), '')



"""Adds pre- and post-latex-views for every object within a book,
mixing in the additional preLatexCode and postLatexCode.
"""

from ftw.book.interfaces import ILaTeXCodeInjectionEnabled
from ftw.book.interfaces import IWithinBookLayer
from ftw.pdfgenerator.view import MakoLaTeXView
from zope.component import adapts
from zope.interface import Interface


class InjectionLaTeXViewBase(MakoLaTeXView):

    def get_rendered_latex_for(self, fieldname):
        field = self.context.Schema().getField(fieldname)

        # in some cases the field can not be retrieved.
        if not field:
            return ''

        code = field.get(self.context)
        if not code:
            return ''

        latex = [
            '',
            '%% ---- LaTeX injection (%s) at %s' % (
                fieldname,
                '/'.join(self.context.getPhysicalPath())),
            code,
            '%% ---- end LaTeX injection (%s)' % fieldname
            ]

        return '\n'.join(latex)


class PreInjectionLaTeXView(InjectionLaTeXViewBase):
    """Mixes in the preLatexCode for every object providing
    ILaTeXCodeInjectionEnabled and within a book.
    """

    adapts(ILaTeXCodeInjectionEnabled, IWithinBookLayer, Interface)

    def render(self):
        return self.get_rendered_latex_for('preLatexCode')


class PostInjectionLaTeXView(InjectionLaTeXViewBase):
    """Mixes in the postLatexCode for every object providing
    ILaTeXCodeInjectionEnabled and within a book.
    """

    adapts(ILaTeXCodeInjectionEnabled, IWithinBookLayer, Interface)

    def render(self):
        return self.get_rendered_latex_for('postLatexCode')



from AccessControl import ClassSecurityInfo
from Products.ATContentTypes import ATCTMessageFactory as _at
from Products.ATContentTypes.content.document import ATDocumentBase
from Products.ATContentTypes.content.schemata import ATContentTypeSchema
from Products.Archetypes import atapi
from ftw.book.config import PROJECTNAME
from ftw.book.interfaces import IHTMLBlock
from simplelayout.base.interfaces import ISimpleLayoutBlock
from simplelayout.types.common.content import simplelayout_schemas
from zope.i18nmessageid import MessageFactory
from zope.interface import implements


_sl = MessageFactory('simplelayout')


htmlblock_schema = ATContentTypeSchema.copy()

htmlblock_schema += atapi.Schema((

        atapi.BooleanField(
            name='showTitle',
            schemata='default',
            default=False,

            widget=atapi.BooleanWidget(
                label=_sl(u'simplelayout_label_showtitle',
                          default=u'Show Title'),
                description=_sl(u'simplelayout_help_showtitle',
                                default=u'Show title'))),

        ))

htmlblock_schema += simplelayout_schemas.textSchema.copy()

htmlblock_schema['text'].widget = atapi.TextAreaWidget(
    label=_at(u'label_body_text', default=u'Body Text'),
    description='',
    rows=32,
    cols=70)

htmlblock_schema['title'].required = False
htmlblock_schema['title'].searchable = 0
htmlblock_schema['excludeFromNav'].default = True
htmlblock_schema['description'].widget.visible = {'edit': 0, 'view': 0}
simplelayout_schemas.finalize_simplelayout_schema(htmlblock_schema)


class HTMLBlock(ATDocumentBase):
    """A simplelayout block providing an HTML text field.
    """

    security = ClassSecurityInfo()
    implements(IHTMLBlock, ISimpleLayoutBlock)
    schema = htmlblock_schema


atapi.registerType(HTMLBlock, PROJECTNAME)



from ftw.book.interfaces import ILaTeXCodeInjectionEnabled
from ftw.book.interfaces import IWithinBookLayer
from ftw.book.testing import FTW_BOOK_INTEGRATION_TESTING
from plone.mocktestcase import MockTestCase
from zope.interface import alsoProvides
from zope.interface import noLongerProvides


class TestLatexInjectionExtender(MockTestCase):

    layer = FTW_BOOK_INTEGRATION_TESTING

    def setUp(self):
        portal = self.layer['portal']

        self.folder = portal.get(portal.invokeFactory('Folder',
                                                      'latex-injection-test'))

        self.page = self.folder.get(
            self.folder.invokeFactory('Page', 'latex-test-page',
                                      title='First page'))

        self.book = self.folder.get(
            self.folder.invokeFactory('Book', 'latex-injection-book',
                                      title='My Book'))

        self.chapter = self.book.get(self.book.invokeFactory(
                'Chapter', 'chapter-one', title='Chapter One'))

    def tearDown(self):
        portal = self.layer['portal']
        portal.manage_delObjects(['latex-injection-test'])

    def test_page_is_enabled_on_all_objects(self):
        self.assertTrue(ILaTeXCodeInjectionEnabled.providedBy(self.folder))
        self.assertTrue(ILaTeXCodeInjectionEnabled.providedBy(self.page))
        self.assertTrue(ILaTeXCodeInjectionEnabled.providedBy(self.book))
        self.assertTrue(ILaTeXCodeInjectionEnabled.providedBy(self.chapter))

    def test_page_has_no_injected_fields(self):
        self.assertEquals(self.page.Schema().getField('preLatexCode'), None)
        self.assertEquals(self.page.Schema().getField('postLatexCode'), None)

    def test_folder_has_no_injected_fields(self):
        self.assertEquals(self.folder.Schema().getField('preLatexCode'), None)
        self.assertEquals(
            self.folder.Schema().getField('postLatexCode'), None)

    def test_book_has_injected_fields(self):
        # Usualy this happend during traversal - check layer.py
        # In this case a browser test could be the better way
        alsoProvides(self.book.REQUEST, IWithinBookLayer)

        self.assertNotEqual(self.book.Schema().getField('preLatexCode'), None)
        self.assertNotEqual(self.book.Schema().getField('postLatexCode'),
                            None)
        # cleanup
        noLongerProvides(self.book.REQUEST, IWithinBookLayer)

    def test_chapter_has_injected_fields(self):
        # Usualy this happend during traversal - check layer.py
        # In this case a browser test could be the better way
        alsoProvides(self.chapter.REQUEST, IWithinBookLayer)

        self.assertNotEqual(self.chapter.Schema().getField('preLatexCode'),
                            None)
        self.assertNotEqual(self.chapter.Schema().getField('postLatexCode'),
                            None)
        # cleanup
        noLongerProvides(self.chapter.REQUEST, IWithinBookLayer)



from ftw.book.interfaces import IBook
from ftw.book.testing import LATEX_ZCML_LAYER
from ftw.pdfgenerator.interfaces import IHTML2LaTeXConverter
from ftw.pdfgenerator.interfaces import ILaTeXLayout
from ftw.pdfgenerator.interfaces import ILaTeXView
from ftw.testing import MockTestCase
from simplelayout.base.interfaces import IBlockConfig
from simplelayout.types.common.interfaces import IParagraph
from zope.component import getMultiAdapter
from zope.interface import alsoProvides


class TestParagraphLaTeXView(MockTestCase):

    layer = LATEX_ZCML_LAYER

    def get_mocks(self):
        # mock is not yet in replay mode, so we use another layout dummy..
        layout_obj = self.create_dummy()
        alsoProvides(layout_obj, ILaTeXLayout)
        converter = getMultiAdapter((object(), object(), layout_obj),
                                    IHTML2LaTeXConverter)

        context = self.providing_stub([IParagraph, IBlockConfig])
        request = self.create_dummy()
        layout = self.providing_mock([ILaTeXLayout])
        self.expect(layout.get_converter()).result(converter).count(0, None)

        return context, request, layout

    def create_image_mocks(self, image_layout, caption, uid):
        context, request, layout = self.get_mocks()

        image = self.create_dummy(get_size=lambda: 11, data='hello world')
        builder = self.mocker.mock()

        self.expect(layout.get_builder()).result(builder)
        self.expect(builder.add_file('%s_image.jpg' % uid, image.data))

        self.expect(layout.use_package('graphicx'))
        self.expect(layout.use_package('wrapfig'))

        self.expect(context.getImage()).result(image).count(1, None)
        self.expect(context.image_layout).result(image_layout)
        self.expect(context.getImageCaption()).result(caption)
        self.expect(context.UID()).result(uid)

        return context, request, layout

    def test_get_image_latex_no_latex_with_no_image(self):
        request = self.create_dummy()
        layout = self.create_dummy()

        context = self.providing_stub([IParagraph])
        self.expect(context.getImage()).result(None)

        self.replay()

        view = getMultiAdapter((context, request, layout), ILaTeXView)
        latex = view.get_image_latex(True)
        self.assertEqual(latex, '')

    def test_get_image_latex_with_layout_noimage(self):
        request = self.create_dummy()
        layout = self.create_dummy()

        context = self.providing_stub([IParagraph, IBlockConfig])

        self.expect(context.getImage()).result(self.create_dummy(
                get_size=lambda: 1))
        self.expect(context.image_layout).result('no-image')
        self.expect(context.getImageCaption()).result('')

        self.replay()

        view = getMultiAdapter((context, request, layout), ILaTeXView)
        latex = view.get_image_latex(True)
        self.assertEqual(latex, '')

    def test_get_text_latex_some_text(self):
        context, request, layout = self.get_mocks()

        self.expect(context.getText()).result(
            ' Hello little world\n')

        self.replay()
        view = getMultiAdapter((context, request, layout), ILaTeXView)
        latex = view.get_text_latex()

        self.assertEqual(latex, 'Hello little world\n')

    def test_get_text_latex_no_text(self):
        context, request, layout = self.get_mocks()

        self.expect(context.getText()).result('')

        self.replay()

        view = getMultiAdapter((context, request, layout), ILaTeXView)
        latex = view.get_text_latex()

        self.assertEqual(latex, '')

    def test_full_latex_rendering(self):
        paragraph, request, layout = self.create_image_mocks(
            'small', 'THE image', '123')

        self.expect(paragraph.getShowTitle()).result(True)
        self.expect(paragraph.pretty_title_or_id()).result(
            'My <b>block</b> title')
        self.expect(paragraph.getText()).result(
            'Thats <b>some</b> text.   ')

        book = self.providing_stub([IBook])
        self.set_parent(paragraph, book)

        self.replay()

        view = getMultiAdapter((paragraph, request, layout), ILaTeXView)
        latex = view.render()

        self.assertIn(r'\chapter{My {\bf block} title}', latex)
        self.assertIn(r'\end{wrapfigure}', latex)
        imagepart, textpart = latex.split(r'\end{wrapfigure}')

        self.assertIn(r'\includegraphics', imagepart)
        self.assertIn('123_image', imagepart)
        self.assertIn(r'Thats {\bf some} text.', textpart)



from simplelayout.types.common.interfaces import IPage
from zope.interface import Interface


class IBook(Interface):
    """Book marker interface.
    """


class IChapter(IPage):
    """Chapter marker interface.
    """


class IHTMLBlock(Interface):
    """HTMLBlock marker interface.
    """


class IRemark(Interface):
    """Remark marker interface.
    """


class ITable(Interface):
    """Remark marker interface.
    """


class IAddRemarkLayer(Interface):
    """ Request layer interface, provided if we select to show remarks in
    the pdf export wizard
    """


class IWithinBookLayer(Interface):
    """Request layer interface, automatically provided by request
    when traversing over book.
    """


class ILaTeXCodeInjectionEnabled(Interface):
    """Enables LaTeX code injection for admins on
    book-objects (chapters, SL-paragraphs).
    """



from zope.publisher.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile


class BookView(BrowserView):

    template = ViewPageTemplateFile('book_view.pt')

    def renderindex(self):
        return self.context.restrictedTraverse('@@index_view')()



from ftw.book.browser.reader import renderer
from ftw.book.browser.reader.interfaces import IBookReaderRenderer
from ftw.testing import MockTestCase
from mocker import ANY
from simplelayout.base.interfaces import ISimpleLayoutBlock
from unittest2 import TestCase
from zope.component import provideAdapter, adaptedBy
from zope.interface import Interface
from zope.traversing.adapters import DefaultTraversable
from zope.traversing.interfaces import ITraversable


class TestBaseBookReaderRenderer(TestCase):

    def test_implements(self):
        self.assertTrue(IBookReaderRenderer.implementedBy(
                renderer.BaseBookReaderRenderer))

    def test_init_sets_attributes(self):
        context = object()
        request = object()
        readerview = object()

        obj = renderer.BaseBookReaderRenderer(
            context, request, readerview)

        self.assertEqual(obj.context, context)
        self.assertEqual(obj.request, request)
        self.assertEqual(obj.readerview, readerview)


class TestDefaultBlcokRenderer(MockTestCase):

    def test_component(self):
        self.assertTrue(IBookReaderRenderer.implementedBy(
                renderer.DefaultBlockRenderer))

        adapt_interfaces = adaptedBy(renderer.DefaultBlockRenderer)
        self.assertEqual(adapt_interfaces[0], ISimpleLayoutBlock)

    def test_render_renders_sl_block(self):
        context = self.stub()
        self.expect(context.restrictedTraverse('block_view')()).result(
            'sl block html')

        self.replay()

        obj = renderer.DefaultBlockRenderer(context, object(), object())
        self.assertEqual(obj.render(), 'sl block html')


class TestBookRenderer(MockTestCase):

    def setUp(self):
        # Page templates use the traversing mechainsm for accessing
        # attributes on objects, so we need to register the default
        # traversable adapter.
        provideAdapter(factory=DefaultTraversable,
                       adapts=[Interface],
                       provides=ITraversable)

    def test_book_render(self):
        response = self.stub()
        self.expect(response.getHeader(ANY)).result('')
        self.expect(response.setHeader(ANY, ANY))

        request = self.stub()
        self.expect(request.debug).result(True)
        self.expect(request.response).result(response)

        context = self.stub()
        self.expect(context.Title()).result('Book')
        self.expect(context.__parent__).result(None)

        self.replay()

        obj = renderer.BookRenderer(context, request, object())
        html = obj.render()
        self.assertIn('<h1>Book</h1>', html)



from ftw.book.interfaces import IWithinBookLayer
from ftw.pdfgenerator import interfaces
from ftw.pdfgenerator.html2latex.converter import HTML2LatexConverter
from zope.component import adapts
from zope.interface import implements, Interface


class BookHTML2LatexConverter(HTML2LatexConverter):
    implements(interfaces.IHTML2LaTeXConverter)
    adapts(Interface, IWithinBookLayer, interfaces.ILaTeXLayout)

    def __init__(self, context, request, layout):
        HTML2LatexConverter.__init__(self, context, request, layout)

        custom_patterns = [
            # requires package "soul", included in book_layout
            (interfaces.HTML2LATEX_MODE_REGEXP,
             r'<span.*?class="[^=]*?visualHighlight[^"]*"[^>]*>(.*?)</span>',
             r'\\hl{\g<1>}'),
            ]

        self.register_patterns(custom_patterns)

        self.layout.use_package('soul')



from AccessControl import ClassSecurityInfo
from Products.ATContentTypes.content.document import ATDocumentBase
from Products.ATContentTypes.content.schemata import ATContentTypeSchema
from Products.Archetypes import atapi
from ftw.book.config import PROJECTNAME
from ftw.book.interfaces import IRemark
from simplelayout.base.interfaces import ISimpleLayoutBlock
from simplelayout.types.common.content import simplelayout_schemas
from zope.i18nmessageid import MessageFactory
from zope.interface import implements


_sl = MessageFactory('simplelayout')

remark_schema = ATContentTypeSchema.copy()

remark_schema += simplelayout_schemas.textSchema.copy()

remark_schema['title'].required = False
remark_schema['title'].searchable = 0
remark_schema['excludeFromNav'].default = True
remark_schema['description'].widget.visible = {'edit': 0, 'view': 0}
simplelayout_schemas.finalize_simplelayout_schema(remark_schema)


class Remark(ATDocumentBase):
    """A simplelayout block used for comments
    """

    security = ClassSecurityInfo()
    implements(IRemark, ISimpleLayoutBlock)
    schema = remark_schema


atapi.registerType(Remark, PROJECTNAME)



from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ftw.book.browser.reader.interfaces import IBookReaderRenderer
from ftw.book.interfaces import IBook
from simplelayout.base.interfaces import ISimpleLayoutBlock
from zope.component import adapts
from zope.interface import implements, Interface
from zope.publisher.interfaces.browser import IBrowserView


class BaseBookReaderRenderer(object):
    """Base block renderer used to subclass.
    """

    implements(IBookReaderRenderer)

    def __init__(self, context, request, readerview):
        self.context = context
        self.request = request
        self.readerview = readerview


class DefaultBlockRenderer(BaseBookReaderRenderer):
    """The simplelayout block renderer.
    It renders the simplealyout default "block_view".
    """

    adapts(ISimpleLayoutBlock, Interface, IBrowserView)

    def render(self):
        view = self.context.restrictedTraverse('block_view')
        return view()


class BookRenderer(BaseBookReaderRenderer):
    """The book renderer renders the title page and the table of
    contents of the book.
    """

    adapts(IBook, Interface, IBrowserView)

    template = ViewPageTemplateFile('templates/book_title.pt')

    def render(self):
        return self.template(title=self.context.Title())



from ZPublisher.BaseRequest import DefaultPublishTraverse
from ftw.book.interfaces import IBook, IWithinBookLayer
from zope.component import adapts
from zope.dottedname.resolve import resolve
from ftw.pdfgenerator.utils import provide_request_layer
from zope.publisher.interfaces import IRequest


class BookTraverse(DefaultPublishTraverse):
    adapts(IBook, IRequest)

    def publishTraverse(self, request, name):

        layout_layer_name = getattr(self.context, 'latex_layout', '')
        if layout_layer_name:
            layout_layer = resolve(layout_layer_name)
            provide_request_layer(request, layout_layer)

        provide_request_layer(request, IWithinBookLayer)

        return super(BookTraverse, self).publishTraverse(request, name)



from Products.Archetypes import atapi
from Products.Archetypes.public import TextField
from archetypes.schemaextender.field import ExtensionField
from archetypes.schemaextender.interfaces import ISchemaExtender
from ftw.book import _
from ftw.book.interfaces import IWithinBookLayer
from ftw.book.interfaces import ILaTeXCodeInjectionEnabled
from zope.component import adapts
from zope.interface import implements


class LaTeXCodeField(ExtensionField, TextField):
    pass


class LaTeXCodeInjectionExtender(object):
    adapts(ILaTeXCodeInjectionEnabled)
    implements(ISchemaExtender)

    fields = []

    fields.append(LaTeXCodeField(
            name='preLatexCode',
            schemata='LaTeX',
            default_content_type='application/x-latex',
            allowable_content_types='application/x-latex',
            write_permission='ftw.book: Modify LaTeX Injection',

            widget=atapi.TextAreaWidget(
                label=_(u'pre_latex_code_label',
                        default=u'LaTeX code above content'),
                description=_(u'pre_latex_code_help',
                              default=u''))))

    fields.append(LaTeXCodeField(
            name='postLatexCode',
            schemata='LaTeX',
            default_content_type='application/x-latex',
            allowable_content_types='application/x-latex',
            write_permission='ftw.book: Modify LaTeX Injection',

            widget=atapi.TextAreaWidget(
                label=_(u'post_latex_code_label',
                        default=u'LaTeX code beneath content'),
                description=_(u'post_latex_code_help',
                              default=u''))))

    def __init__(self, context):
        self.context = context

    def getFields(self):
        if not self._context_is_within_book():
            return []

        return self.fields

    def _context_is_within_book(self):

        # In some cases REQUEST is no available.
        # XXX: This is a quick fix without debugging, just a guess
        if not hasattr(self.context, 'REQUEST'):
            return False

        if IWithinBookLayer.providedBy(self.context.REQUEST):
            return True
        return False



from BeautifulSoup import BeautifulSoup
from ftw.book.table.calculator import ColumnWidthsCalculator
from ftw.book.table.tablepart import TablePartBody
from ftw.book.table.tablepart import TablePartFooter
from ftw.book.table.tablepart import TablePartHeader
from ftw.pdfgenerator.utils import html2xmlentities
from xml.dom import minidom
import re


BORDER_STYLE_MAPPING = {
    'grid': ['raw-table'],
    'invisible': ['invisible'],
    'fancy_listing': ['raw-table'],
}


class TableGenerator(object):

    def __init__(self, context):
        self.context = context
        self.doc = None
        self.table_node = None
        self._activeColumns = None
        self._columnProperties = None

    def render(self):
        if not self.active_columns:
            return ''

        if not self.context.getData():
            return ''

        self.doc = minidom.Document()
        self.table_node = self.create_table_element()
        self.create_caption()
        self.create_colgroup()
        self.create_table_head()
        self.create_table_body()
        self.create_table_footer()
        # render
        html = self.doc.toxml(encoding='utf8')
        # try to remove xml definition
        pattern = re.compile('^<\?xml .*?\?>')
        try:
            html = pattern.sub('', html)
        except TypeError:
            pass
        return html

    def create_table_element(self):
        """ Create the table
        """
        css_classes = ['notListed']

        css_classes += BORDER_STYLE_MAPPING.get(
            self.context.getBorderLayout(), '')

        if self.context.getNoLifting():
            css_classes.append('no-lifting')

        attrs = {
                'summary': self.context.Description(),
                'class': ' '.join(css_classes),
        }
        return self._create_node('table', self.doc, **attrs)

    def create_caption(self):
        """ Create the caption node object if we want to show the title
        """
        if not self.context.getShowTitle():
            return

        title = self.context.Title()
        if isinstance(title, str):
            title = title.decode('utf-8')

        self._create_node('caption', self.table_node, title)

    def create_colgroup(self):
        """ Create the colgroup and calculate the correct width
        """
        colgroup = self._create_node('colgroup', self.table_node)
        calculator = ColumnWidthsCalculator()

        widths = calculator(self.column_widths)

        # create colgroup
        for i in range(len(self.active_column_names)):
            self._create_node('col', colgroup, content=False, **{
                    'width': '%i%%' % widths[i],
            })

        return colgroup

    def create_table_head(self):
        """ Create the head of the table based on the setted header-rows
        in the headerRows-field
        """
        num_header_rows = self.context.getHeaderRows(as_int=True)
        if num_header_rows == 0:
            return None

        # Table head
        thead = self._create_node('thead', self.table_node)
        rows = self.context.getData()[:num_header_rows]

        tablepart = TablePartHeader(
            rows,
            thead,
            0,
            self.active_column_names,
            self.context.getFirstColumnIsHeader(),
            self.context.getBorderLayout(),
        )

        self._create_rows(tablepart)

        return thead

    def create_table_body(self):
        """ Create the table body rows
        """
        num_header_rows = self.context.getHeaderRows(as_int=True)
        num_footer_rows = self.context.getFooterRows(as_int=True)
        available_rows = len(self.context.getData())
        num_body_rows = available_rows - (num_header_rows + num_footer_rows)

        if num_body_rows <= 0:
            return None

        tbody = self._create_node('tbody', self.table_node)
        body_begins_at = available_rows - (num_body_rows + num_footer_rows)
        rows = self.context.getData()[
            num_header_rows:len(self.context.getData()) - num_footer_rows]

        tablepart = TablePartBody(
            rows,
            tbody,
            body_begins_at,
            self.active_column_names,
            self.context.getFirstColumnIsHeader(),
            self.context.getBorderLayout(),
        )

        self._create_rows(tablepart)

        return tbody

    def create_table_footer(self):
        """ Create the table footer rows
        """
        num_footer_rows = self.context.getFooterRows(as_int=True)
        if num_footer_rows == 0:
            return None

        footer = self._create_node('tfoot', self.table_node)
        footer_begins_at = len(self.context.getData()) - num_footer_rows
        rows = self.context.getData()[-num_footer_rows:]

        tablepart = TablePartFooter(
            rows,
            footer,
            footer_begins_at,
            self.active_column_names,
            self.context.getFirstColumnIsHeader(),
            self.context.getBorderLayout(),
            self.context.getFooterIsBold(),
        )

        self._create_rows(tablepart)

        return footer

    @property
    def active_columns(self):
        """ Return number of active columns as int
        """
        return len(self.active_column_names)

    @property
    def active_column_names(self):
        """ Return the names of all active columns in a list
        """
        if self._activeColumns is None:
            self._activeColumns = [column['columnId'] for column in \
                self.context.getColumnProperties() if column['active']]
        return self._activeColumns

    @property
    def column_widths(self):
        """ Return the columnwidths in a list
        """
        column_widths = []
        for name in self.active_column_names:
            column_widths.append(self._get_column_properties(name)['width'])
        return column_widths

    def _get_column_properties(self, columnName):
        """ Return the proberties of all columns in a dict.
        key: columnId
        value: properties
        """
        if self._columnProperties is None:
            self._columnProperties = {}
            for column in self.context.getColumnProperties():
                self._columnProperties[column['columnId']] = column
        return self._columnProperties[columnName]

    def _set_column_width(self, widths, width, width_condition=False):
        """ Set the column-widths in the widths
        """
        for key in widths.keys():
            if width_condition is False or widths[key] == width_condition:
                widths[key] = width

        return widths

    def _clean_and_parse_html(html):
        """ Cleanup the given html and parse it
        """
        html = str(BeautifulSoup(html))
        try:
            doc = minidom.parseString('<data>%s</data>' % html)
        except UnicodeEncodeError:
            doc = minidom.parseString('<data>FEHLER</data>')
        return doc
    _clean_and_parse_html = staticmethod(_clean_and_parse_html)

    def _create_rows(self, tablepart):
        """ Create rows with the given tablepart object
        """
        for i, row in enumerate(tablepart.rows):
            tablepart.set_row_node(
                self._create_node(tablepart.get_row_type(), tablepart.parent))

            for j, col_name in enumerate(tablepart.column_names):

                tablepart.set_is_first_cell(j == 0 and True or False)

                cell = self._create_cell(
                    row, col_name, i + tablepart.begin_at, tablepart)

                if 'fullColspan' in self._get_css(
                    i + tablepart.begin_at, col_name, tablepart):

                    # Just set the first cell, then we brak
                    cell.setAttribute(
                        'colspan', str(len(tablepart.column_names)))
                    break

    def _create_cell(self, row, col_name, row_num, tablepart):
        """ Create a cell in a row with
        """
        attrs = {}
        attrs['class'] = ' '.join(self._get_css(row_num, col_name, tablepart))

        # Add additional attrs
        attrs.update(tablepart.get_additional_attrs(row_num, col_name))

        return self._create_node(
            tablepart.get_cell_type(),
            tablepart.get_row_node(),
            tablepart.wrap_text_in_attr(row[col_name]),
            **attrs)

    def _get_css(self, row_num, col_name, tablepart):
        """ Return global css and css generated from a tablepart object
        """
        css = []

        # Add available column css classes
        col = self._get_column_properties(col_name)
        if col['alignment']:
            css.append(col['alignment'])
        if col['bold']:
            css.append('bold')
        if col['indent']:
            css.append(col['indent'])

        # Add available row css classes
        css += self.context.getData()[row_num].get('row_format', '').split(' ')

        return tablepart.get_css(css, row_num, col_name)

    def _create_node(self, tag_name, parent_node, content='', **kwargs):
        """ Create a new minidom node
        """
        node = self.doc.createElement(tag_name)

        for key, value in kwargs.items():
            if isinstance(value, str):
                value = value.decode('utf8')
            node.setAttribute(key, value)

        if content != False:
            if not content:
                content = ' '

            if isinstance(content, unicode):
                content = content.encode('utf8')

            content = html2xmlentities(content)
            contentDoc = TableGenerator._clean_and_parse_html(content)
            contentNode = contentDoc.getElementsByTagName('data')[0]

            for elm in list(contentNode.childNodes):
                node.appendChild(elm)

        parent_node.appendChild(node)

        return node



from ftw.book.helpers import BookHelper
from ftw.book.interfaces import IBook
from plone.mocktestcase import MockTestCase
from Products.CMFCore.interfaces._content import IFolderish
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from zope.interface import directlyProvides


class TestUnitBookHelper(MockTestCase):
    """ Unittests for helper methods in BookHelper Class
    """

    def setUp(self):
        """ Setup test environment

        book
        |
        --- chapter1
        |
        --- chapter2
            |
            ---chapter3
            |  |
            |  ---chapter5
            |  |
            |  ---image
            |  |
            |  ---paragraph1
            |  |
            |  ---paragraph2
            |
            ---chapter4
        """

        # Helper object
        self.helper = BookHelper()

        #  Siteroot
        self.root = self.create_dummy()
        directlyProvides(self.root, IPloneSiteRoot)

        #  Book
        self.book = self.create_dummy()
        directlyProvides(self.book, [IBook, IFolderish])
        self.book = self.mocker.proxy(self.book, spec=False, count=False)
        self.expect(self.book.__parent__).result(self.root)
        self.expect(self.book.Title()).result('Book')

        # Childs of book
        self.chapter1 = self.create_dummy()
        directlyProvides(self.chapter1, IFolderish)
        self.chapter1 = self.mocker.proxy(
            self.chapter1, spec=False, count=False)
        self.expect(self.chapter1.__parent__).result(self.book)
        self.expect(self.chapter1.Title()).result('Chapter1')
        self.expect(self.chapter1.portal_type).result('Chapter')
        self.chapter2 = self.create_dummy()
        directlyProvides(self.chapter2, IFolderish)
        self.chapter2 = self.mocker.proxy(
            self.chapter2, spec=False, count=False)
        self.expect(self.chapter2.__parent__).result(self.book)
        self.expect(self.chapter2.Title()).result('Chapter2')
        self.expect(self.chapter2.portal_type).result('Chapter')

        #  Childs of chapter 2
        self.chapter3 = self.create_dummy()
        directlyProvides(self.chapter3, IFolderish)
        self.chapter3 = self.mocker.proxy(
            self.chapter3, spec=False, count=False)
        self.expect(self.chapter3.__parent__).result(self.chapter2)
        self.expect(self.chapter3.Title()).result('Chapter3')
        self.expect(self.chapter3.portal_type).result('Chapter')
        self.chapter4 = self.create_dummy()
        directlyProvides(self.chapter4, IFolderish)
        self.chapter4 = self.mocker.proxy(
            self.chapter4, spec=False, count=False)
        self.expect(self.chapter4.__parent__).result(self.chapter2)
        self.expect(self.chapter4.Title()).result('Chapter4')
        self.expect(self.chapter4.portal_type).result('Chapter')

        #  Childs of chapter 3
        self.chapter5 = self.create_dummy()
        directlyProvides(self.chapter5, IFolderish)
        self.chapter5 = self.mocker.proxy(
            self.chapter5, spec=False, count=False)
        self.expect(self.chapter5.__parent__).result(self.chapter3)
        self.expect(self.chapter5.Title()).result('Chapter5')
        self.expect(self.chapter5.portal_type).result('Chapter')
        self.image = self.mocker.mock(count=False)
        self.expect(self.image.__parent__).result(self.chapter3)
        self.expect(self.image.Title()).result('Image')
        self.expect(self.image.portal_type).result('Image')
        self.paragraph1 = self.mocker.mock(count=False)
        self.expect(self.paragraph1.__parent__).result(self.chapter3)
        self.expect(self.paragraph1.showTitle).result(True)
        self.expect(self.paragraph1.Title()).result('Paragraph1')
        self.expect(self.paragraph1.portal_type).result('Paragraph')
        self.paragraph2 = self.mocker.mock(count=False)
        self.expect(self.paragraph2.__parent__).result(self.chapter3)
        self.expect(self.paragraph2.__inner__).result(self.paragraph2)
        self.expect(self.paragraph2.showTitle).result(True)
        self.expect(self.paragraph2.Title()).result('Paragraph2')
        self.expect(self.paragraph2.title_or_id()).result('Paragraph2')
        self.expect(self.paragraph2.portal_type).result('Paragraph')

        # Folderish content
        self.expect(self.book.contentValues()).result(
            [self.chapter1, self.chapter2])
        self.expect(self.chapter1.contentValues()).result([])
        self.expect(self.chapter2.contentValues()).result(
            [self.chapter3, self.chapter4])
        self.expect(self.chapter3.contentValues()).result(
            [self.chapter5, self.image, self.paragraph1, self.paragraph2])
        self.expect(self.chapter4.contentValues()).result([])
        self.expect(self.chapter5.contentValues()).result([])

    def test_generate_title(self):
        """ Test generate title
        """
        self.replay()
        title = self.helper(self.paragraph2)
        self.assertTrue(title == '<h4>2.1.3 Paragraph2</h4>')

    def test_generate_valid_hierarchy_h_tags(self):
        """ Test generate valid hierarchy h tags
        """
        self.replay()
        tags = self.helper.generate_valid_hierarchy_h_tags(self.chapter3)
        self.assertTrue(
            tags == '<h1>Book</h1>' +
                    '<h2>Chapter2</h2>' +
                    '<h3>Chapter3</h3>')

    def test_get_folder_position(self):
        """ Test get folder position
        """
        self.replay()
        position = self.helper.get_folder_position(self.paragraph2)
        self.assertTrue(position == 3)

    def test_get_hierarchy_position(self):
        """ Test hierarchy position
        """
        self.replay()
        position = self.helper.get_hierarchy_position(self.paragraph2)
        self.assertTrue(position == 4)



from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ftw.book.browser.toc_tree import BookTocTree
from plone.app.layout.navigation.navtree import buildFolderTree
from zope.publisher.browser import BrowserView


class IndexView(BrowserView):

    template = ViewPageTemplateFile('index_view.pt')

    def __init__(self, *args, **kwargs):
        super(IndexView, self).__init__(*args, **kwargs)
        self.tree = None

    def __call__(self):
        context = self.context
        query = {
            'path': '/'.join(context.getPhysicalPath())}
        raw_tree = buildFolderTree(context, obj=context, query=query)
        toc_tree = BookTocTree()
        tree = toc_tree(raw_tree)
        self.tree = tree
        return self.template()



from ftw.book.interfaces import IBook
from ftw.book.content.book import Book
from plone.mocktestcase import MockTestCase
from zope.interface import directlyProvides
from Products.Archetypes.event import ObjectInitializedEvent
from ftw.book import eventhandler
from plone.portlets.interfaces import IPortletManager
from plone.portlets.interfaces import IPortletAssignmentMapping
from plone.portlets.interfaces import ILocalPortletAssignmentManager
from mocker import ANY
from zope.interface import Interface


class TestEventhandler(MockTestCase):
    
    def test_events(self):
        request = self.create_dummy()
        manager = self.mocker.mock(count=False)
        self.expect(manager()).result(None)
        portal_url = self.mocker.mock(count=False)
        self.expect(portal_url.getPortalPath()).result(('','plone'))
        assignable = self.mocker.mock(count=False)
        mapping = self.mocker.mock(count=False)
        pers_mapping = dict()
        parent = self.create_dummy(title='parent',absolute_url=lambda:'http://nohost/plone/parent')
        book = self.mocker.mock(Book, count=False)
        self.expect(assignable.setBlacklistStatus(ANY, ANY)).result(None)
        self.expect(book.title).result("mybook")
        self.expect(book.aq_parent).result(parent)
        self.expect(book.getPhysicalPath()).result(('','plone','parent', 'mybook'))
        self.expect(book()).result(None)
        self.expect(book.portal_url).result(portal_url)
        directlyProvides(book, IBook)
        self.mock_utility(manager, IPortletManager, name="plone.leftcolumn")
        self.mock_utility(manager, IPortletManager, name="plone.rightcolumn")
        self.mock_adapter(mapping, IPortletAssignmentMapping, (Interface, Interface))
        self.mock_adapter(assignable, ILocalPortletAssignmentManager, (Interface, Interface))
        self.expect(assignable(ANY, ANY)).result(assignable)
        self.expect(mapping(ANY, ANY)).result(mapping)
        self.expect(mapping.__of__(ANY)).result(pers_mapping)
        self.replay()
        event = ObjectInitializedEvent(book, request)
        eventhandler.add_navigation_portlet(book, event)
        gtpp_ass = pers_mapping['go-to-parent-portlet']
        nav_ass = pers_mapping['navigation']
        self.assertEqual(gtpp_ass.title, 'Go To Parent Portlet')
        self.assertEqual(nav_ass.__dict__, {'name': u'', 'bottomLevel': 0, 'topLevel': 0,
         'currentFolderOnly': False, 'includeTop': 1, 'root': 'lone/parent/mybook'})
        self.assertEqual(nav_ass.title, u'Navigation')




from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ftw.book.helpers import BookHelper
from simplelayout.base import viewlets


class SimpleLayoutListingViewlet(viewlets.SimpleLayoutListingViewlet):

    render = ViewPageTemplateFile('listing.pt')
    helper = BookHelper()

    def get_valid_parent_h_tags(self):
        return self.helper.generate_valid_hierarchy_h_tags(self.context)



from ftw.book.interfaces import IWithinBookLayer
from ftw.book.latex.converter import BookHTML2LatexConverter
from ftw.book.testing import LATEX_ZCML_LAYER
from ftw.pdfgenerator.interfaces import IHTML2LaTeXConverter
from ftw.pdfgenerator.interfaces import ILaTeXLayout
from ftw.testing import MockTestCase
from mocker import ANY
from zope.component import queryMultiAdapter
from zope.interface.verify import verifyClass


class TestBookHTML2LatexConverter(MockTestCase):

    layer = LATEX_ZCML_LAYER

    def test_component_is_registered(self):
        context = object()
        request = self.providing_stub([IWithinBookLayer])
        layout = self.providing_stub([ILaTeXLayout])

        self.expect(layout.use_package('soul'))

        self.replay()
        component = queryMultiAdapter((context, request, layout),
                                      IHTML2LaTeXConverter)

        self.assertNotEquals(component, None)
        self.assertEqual(type(component), BookHTML2LatexConverter)
        verifyClass(IHTML2LaTeXConverter, BookHTML2LatexConverter)

    def test_component_registered_only_within_book(self):
        context = object()
        request = object()
        layout = self.providing_stub([ILaTeXLayout])

        self.replay()
        component = queryMultiAdapter((context, request, layout),
                                      IHTML2LaTeXConverter)

        self.assertNotEquals(component, None)
        self.assertNotEqual(type(component), BookHTML2LatexConverter)

    def test_converter_inherits_default_patterns(self):
        context = request = self.create_dummy()
        layout = self.stub()
        self.expect(layout.use_package(ANY))

        self.replay()

        conv = BookHTML2LatexConverter(context, request, layout)

        self.assertEqual(conv.convert('Hello <b>World</b>!'),
                         'Hello {\\bf World}!')

    def test_converter_converts_visualHighlight(self):
        context = request = self.create_dummy()
        layout = self.mocker.mock()
        self.expect(layout.use_package('soul'))

        self.replay()

        conv = BookHTML2LatexConverter(context, request, layout)

        self.assertEqual(
            conv.convert(r'foo <span class="visualHighlight">bar</span> baz'),
            r'foo \hl{bar} baz')

        self.assertEqual(
            conv.convert(r'foo <span id="myid" class="visualHighlight">'
                         r'bar</span> baz'),
            r'foo \hl{bar} baz')

        self.assertEqual(
            conv.convert(r'foo <span class="visualHighlight" id="myid">'
                         r'bar</span> baz'),
            r'foo \hl{bar} baz')

        self.assertEqual(
            conv.convert(r'foo <span class="othercls visualHighlight">'
                         r'bar</span> baz'),
            r'foo \hl{bar} baz')

        self.assertEqual(
            conv.convert(r'foo <span class="visualHighlight othercls">'
                         r'bar</span> baz'),
            r'foo \hl{bar} baz')

        self.assertEqual(
            conv.convert(r'foo <span class="visualHighlight othercls">'
                         r'bar <b>BAR</b> bar</span> baz'),
            r'foo \hl{bar {\bf BAR} bar} baz')

        self.assertEqual(
            conv.convert(r'foo <span class="othercls" id="visualHighlight">'
                         r'bar</span> baz'),
            r'foo bar baz')
